'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports['default'] = connect;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

function _inherits(subClass, superClass) { if (typeof superClass !== 'function' && superClass !== null) { throw new TypeError('Super expression must either be null or a function, not ' + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

require('whatwg-fetch');

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _utilsIsPlainObject = require('../utils/isPlainObject');

var _utilsIsPlainObject2 = _interopRequireDefault(_utilsIsPlainObject);

var _utilsDeepValue = require('../utils/deepValue');

var _utilsDeepValue2 = _interopRequireDefault(_utilsDeepValue);

var _PromiseState = require('../PromiseState');

var _PromiseState2 = _interopRequireDefault(_PromiseState);

var _hoistNonReactStatics = require('hoist-non-react-statics');

var _hoistNonReactStatics2 = _interopRequireDefault(_hoistNonReactStatics);

var _invariant = require('invariant');

var _invariant2 = _interopRequireDefault(_invariant);

var defaultMapPropsToRequestsToProps = function defaultMapPropsToRequestsToProps() {
  return {};
};

function getDisplayName(WrappedComponent) {
  return WrappedComponent.displayName || WrappedComponent.name || 'Component';
}

// Helps track hot reloading.
var nextVersion = 0;

function connect(mapPropsToRequestsToProps) {
  var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

  var finalMapPropsToRequestsToProps = mapPropsToRequestsToProps || defaultMapPropsToRequestsToProps;
  var _options$withRef = options.withRef;
  var withRef = _options$withRef === undefined ? false : _options$withRef;

  // Helps track hot reloading.
  var version = nextVersion++;

  function computeMappings(props) {
    var rawMappings = finalMapPropsToRequestsToProps(props) || {};

    _invariant2['default'](_utilsIsPlainObject2['default'](rawMappings), '`mapPropsToRequestsToProps` must return an object. Instead received %s.', rawMappings);

    var mappings = {};
    Object.keys(rawMappings).forEach(function (prop) {
      mappings[prop] = coerceMapping(prop, rawMappings[prop]);
    });
    return mappings;
  }

  function coerceMapping(prop, mapping) {
    if (Array.isArray(mapping)) {
      return Object.assign({ request: coerceRequest(prop, mapping[0]) }, coerceOpts(prop, mapping[1]));
    } else {
      return { request: coerceRequest(prop, mapping) };
    }
  }

  function coerceRequest(prop, stringOrRequest) {
    if (typeof stringOrRequest === 'string') {
      return new window.Request(stringOrRequest, { credentials: 'same-origin' });
    } else if (stringOrRequest instanceof window.Request) {
      return stringOrRequest;
    } else {
      _invariant2['default'](false, 'Value of first argument of `%s` must be either a string or Request. Instead received %s', prop, stringOrRequest);
    }
  }

  function coerceOpts(prop, opts) {
    if (opts && !_utilsIsPlainObject2['default'](opts)) {
      _invariant2['default'](false, 'Value of second argument of `%s` must be a plain object. Instead received %s', prop, opts);
    }
    return opts || {};
  }

  function handleResponse(response) {
    if (response.status >= 200 && response.status < 300) {
      return response.json();
    } else {
      return response.json().then(function (errorJson) {
        var id = errorJson.id;
        var error = errorJson.error;
        var message = errorJson.message;

        if (error) {
          throw new Error(error, id);
        } else if (message) {
          throw new Error(message, id);
        } else {
          throw new Error(errorJson);
        }
      });
    }
  }

  return function wrapWithConnect(WrappedComponent) {
    var RefetchConnect = (function (_Component) {
      _inherits(RefetchConnect, _Component);

      function RefetchConnect(props, context) {
        _classCallCheck(this, RefetchConnect);

        _Component.call(this, props, context);
        this.version = version;
        this.state = { mappings: {}, data: {}, refreshTimeouts: {} };
      }

      RefetchConnect.prototype.componentWillMount = function componentWillMount() {
        this.refetchData();
      };

      RefetchConnect.prototype.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
        this.refetchData(nextProps);
      };

      RefetchConnect.prototype.componentWillUnmount = function componentWillUnmount() {
        this.clearAllRefreshTimeouts();
      };

      RefetchConnect.prototype.render = function render() {
        var ref = withRef ? 'wrappedInstance' : null;
        return _react2['default'].createElement(WrappedComponent, _extends({}, this.state.data, this.props, { ref: ref }));
      };

      RefetchConnect.prototype.getWrappedInstance = function getWrappedInstance() {
        _invariant2['default'](withRef, 'To access the wrapped instance, you need to specify ' + '{ withRef: true } as the fourth argument of the connect() call.');

        return this.refs.wrappedInstance;
      };

      RefetchConnect.prototype.refetchData = function refetchData() {
        var _this = this;

        var props = arguments.length <= 0 || arguments[0] === undefined ? this.props : arguments[0];

        var nextMappings = computeMappings(props);
        Object.keys(nextMappings).forEach(function (prop) {
          var prev = _this.state.mappings[prop];
          var next = nextMappings[prop];
          var comp = ['request.url', 'request.method', 'refreshInterval'];
          var same = comp.every(function (c) {
            return _utilsDeepValue2['default'](prev, c) === _utilsDeepValue2['default'](next, c);
          });

          if (!same) {
            _this.refetchDatum(prop, next);
          }
        });
      };

      RefetchConnect.prototype.refetchDatum = function refetchDatum(prop, mapping, refreshing) {
        var _this2 = this;

        if (this.state.refreshTimeouts[prop]) {
          window.clearTimeout(this.state.refreshTimeouts[prop]);
        }

        this.setAtomicState(prop, mapping, new _PromiseState2['default']({
          pending: !refreshing,
          refreshing: refreshing,
          value: refreshing ? this.state.data[prop].value : null
        }), null);

        window.fetch(mapping.request.clone()).then(handleResponse).then(function (value) {
          var refreshTimeout = null;
          if (mapping.refreshInterval > 0) {
            refreshTimeout = window.setTimeout(function () {
              _this2.refetchDatum(prop, mapping, true);
            }, mapping.refreshInterval);
          }
          _this2.setAtomicState(prop, mapping, new _PromiseState2['default']({
            fulfilled: true,
            value: value
          }), refreshTimeout);
        })['catch'](function (error) {
          _this2.setAtomicState(prop, mapping, new _PromiseState2['default']({
            rejected: true,
            reason: error
          }), null);
        });
      };

      RefetchConnect.prototype.setAtomicState = function setAtomicState(prop, mapping, promiseState, refreshTimeout) {
        this.setState(function (prevState) {
          var _Object$assign, _Object$assign2, _Object$assign3;

          return {
            mappings: Object.assign(prevState.mappings, (_Object$assign = {}, _Object$assign[prop] = mapping, _Object$assign)),
            data: Object.assign(prevState.data, (_Object$assign2 = {}, _Object$assign2[prop] = promiseState, _Object$assign2)),
            refreshTimeouts: Object.assign(prevState.refreshTimeouts, (_Object$assign3 = {}, _Object$assign3[prop] = refreshTimeout, _Object$assign3))
          };
        });
      };

      RefetchConnect.prototype.clearAllRefreshTimeouts = function clearAllRefreshTimeouts() {
        var _this3 = this;

        Object.keys(this.state.refreshTimeouts).forEach(function (prop) {
          clearTimeout(_this3.state.refreshTimeouts[prop]);
        });
      };

      return RefetchConnect;
    })(_react.Component);

    RefetchConnect.displayName = 'Refetch.connect(' + getDisplayName(WrappedComponent) + ')';
    RefetchConnect.WrappedComponent = WrappedComponent;

    if (process.env.NODE_ENV !== 'production') {
      RefetchConnect.prototype.componentWillUpdate = function componentWillUpdate() {
        if (this.version === version) {
          return;
        }

        // We are hot reloading!
        this.version = version;
        this.clearAllRefreshTimeouts();
        this.refetchData();
      };
    }

    return _hoistNonReactStatics2['default'](RefetchConnect, WrappedComponent);
  };
}

module.exports = exports['default'];